// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you may not use this file except in
// compliance with the License.  You may obtain a copy of the License
// at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and
// limitations under the License.
//
// The Original Code is
// "Michael C. Lee, Jr's small example of the many features of Java3D".
//
// The Initial Developer of the Original Code is GameSpawn, Inc.
// Portions created by GameSpawn, Inc. are
// Copyright (C) 2000 GameSpawn, Inc.  All Rights Reserved.
//
// Contributor(s): 

package com.gamespawn.lab.mikelee.j3d.geometry;

import javax.media.j3d.*;
import javax.vecmath.*;

/**
* @author
*   Michael C. Lee, Jr. (mike@thoselees.com, http://www.thoselees.com/mike)
*/
public class Diamond extends Shape3D
{
    // Math to determine placement of vertecies
    private static final float s3 = (float) Math.sqrt(3.0);
    private static final float s3_3 = s3 / 3.0f;
    private static final float s24_3 = (float) Math.sqrt(24.0) / 3.0f;

    // calculate the center y coordinate
    private static final float ycenter = 0.5f * s24_3;
    // calculate the center y coordinate
    private static final float zcenter = -s3_3;

    // set the 5 points of the diamont
    private static final Point3f side1 = new Point3f(-1.0f, -ycenter, -zcenter);
    private static final Point3f side2 = new Point3f(1.0f, -ycenter, -zcenter);
    private static final Point3f side3 = new Point3f(0.0f, -ycenter, -s3 - zcenter);
    private static final Point3f top = new Point3f(0.0f, s24_3 - ycenter, 0.0f);
    private static final Point3f bottom = new Point3f(0.0f, - ycenter - s24_3, 0.0f);

    // generate the vertecies
    private static final Point3f[] verts = {
	side1, side2, top, // top front face
	side1, top, side3, // top side face
	side2, side3, top, // top side face
    side1, side3, bottom, // bottom front face
    side1, bottom, side3, // bottom side face
    side2, side3, bottom, // bottom side face
    };

    private Point2f texCoord[] =
    {
        new Point2f(0.0f, 0.0f),
	    new Point2f(1.0f, 0.0f),
        new Point2f(0.5f, s3 / 2.0f),
    };

    public Diamond()
    {
    	// this is to build the triangles that form the diamond
        TriangleArray diamond = new TriangleArray(18, TriangleArray.COORDINATES |
	    	TriangleArray.NORMALS | TriangleArray.TEXTURE_COORDINATE_2);

    	diamond.setCoordinates(0, verts);
        for (int loop = 0; loop < 18; loop++)
        {
            diamond.setTextureCoordinate(loop, texCoord[loop%3]);
        }

	    Vector3f normal = new Vector3f();
    	Vector3f v1 = new Vector3f();
	    Vector3f v2 = new Vector3f();
    	Point3f [] pts = new Point3f[3];

	    for (int loop = 0; loop < 3; loop++) pts[loop] = new Point3f();

    	for (int face = 0; face < 6; face++)
        {
	        diamond.getCoordinates(face*3, pts);
	        v1.sub(pts[1], pts[0]);
    	    v2.sub(pts[2], pts[0]);
	        normal.cross(v1, v2);
	        normal.normalize();
    	    for (int loop = 0; loop < 3; loop++)
            {
	    	    diamond.setNormal((face * 3 + loop), normal);
	        }
    	}
	    setGeometry(diamond);
    	setAppearance(new Appearance());
    } // end Diamond constructor
}
